﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;

namespace IANATimezonesConverter
{
    public static class TimeZones
    {
        #region Classes
        [XmlRoot("mapZone")]
        public class IANATimeZone
        {
            [XmlAttribute("other")]
            public string Other { get; set; }

            [XmlAttribute("territory")]
            public string Territory { get; set; }

            [XmlAttribute("type")]
            public string Type { get; set; }
        } // IANATimeZone

        [XmlRoot("mapTimezones")]
        public class MapTimezones
        {
            [XmlAttribute("otherVersion")]
            public string OtherVersion { get; set; }

            [XmlAttribute("typeVersion")]
            public string TypeVersion { get; set; }

            [XmlElement("mapZone")]
            public List<IANATimeZone> Zones { get; set; }
        } // MapTimezones

        [XmlRoot("windowsZones")]
        public class WindowsZones
        {
            [XmlElement("mapTimezones")]
            public MapTimezones MapTimezones { get; set; }
        } // WindowsZones

        /// <summary>
        /// Timezones data from http://unicode.org/repos/cldr/trunk/common/supplemental/windowsZones.xml
        /// </summary>
        [XmlRoot("supplementalData")]
        public class IANATimeZones
        {
            [XmlElement("windowsZones")]
            public WindowsZones WindowsZones { get; set; }
        } // IANATimeZones
        #endregion

        #region Private members
        static IANATimeZones m_IANATimezones = null;
        #endregion

        /// <summary>
        /// Get the windows timezone corresponding to the given IANA one
        /// </summary>
        /// <param name="iANATimezone">The IANA timezone</param>
        /// <returns>The windows timezone name or string.empty</returns>
        public static string GetWindowsTimezone(string iANATimezone)
        {
            lock (typeof(IANATimeZones)) {
                if (m_IANATimezones == null) {
                    Assembly assembly = Assembly.GetExecutingAssembly();
                    using (StreamReader sr = new StreamReader(assembly.GetManifestResourceStream("IANATimezonesConverter.Data.windowsZones.xml"))) {
                        m_IANATimezones = Deserialize<IANATimeZones>(sr.BaseStream);
                    }
                }
            }

            if (m_IANATimezones != null && m_IANATimezones.WindowsZones != null && m_IANATimezones.WindowsZones.MapTimezones != null && m_IANATimezones.WindowsZones.MapTimezones.Zones != null) {
                iANATimezone = iANATimezone.ToLower();
                foreach (IANATimeZone it in m_IANATimezones.WindowsZones.MapTimezones.Zones) {
                    string[] types = it.Type.ToLower().Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);
                    if (types.Contains(iANATimezone))
                        return it.Other;
                }
            }
            return string.Empty;
        } // GetWindowsTimezone

        private static T Deserialize<T>(Stream stream)
        {
            try {
                XmlSerializer serializer = new XmlSerializer(typeof(T));
                T result;
                using (StreamReader sr = new StreamReader(stream, Encoding.UTF8)) {
                    result = (T)serializer.Deserialize(sr);
                }
                return result;
            } catch (Exception ex) {
                System.Diagnostics.Debug.WriteLine(string.Format("Error deserializing object: {0}", ex.Message));
                if (ex.InnerException != null)
                    System.Diagnostics.Debug.WriteLine(ex.InnerException.Message);
            }
            return default(T);
        } // Deserialize

    }
}
